@@ -1,8 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2013, 2015  Université de Bordeaux
- * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -30,7 +29,7 @@
 #include <common/blas.h>
 
 #ifdef STARPU_USE_CUDA
-#include <cublas.h>
+#include <starpu_cublas_v2.h>
 #endif
 
 #include "axpy.h"
@@ -52,7 +51,7 @@ TYPE _alpha = 3.41;
 /* descriptors for StarPU */
 starpu_data_handle_t _handle_y, _handle_x;
 
-void axpy_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
+void axpy_cpu(void *descr[], void *arg)
 {
 	TYPE alpha = *((TYPE *)arg);
 
@@ -65,7 +64,7 @@ void axpy_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
 }
 
 #ifdef STARPU_USE_CUDA
-void axpy_gpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
+void axpy_gpu(void *descr[], void *arg)
 {
 	TYPE alpha = *((TYPE *)arg);
 
@@ -74,8 +73,9 @@ void axpy_gpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
 	TYPE *block_x = (TYPE *)STARPU_VECTOR_GET_PTR(descr[0]);
 	TYPE *block_y = (TYPE *)STARPU_VECTOR_GET_PTR(descr[1]);
 
-	CUBLASAXPY((int)n, alpha, block_x, 1, block_y, 1);
-	cudaStreamSynchronize(starpu_cuda_get_local_stream());
+	cublasStatus_t status = CUBLASAXPY(starpu_cublas_get_local_handle(), (int)n, &alpha, block_x, 1, block_y, 1);
+	if (status != CUBLAS_STATUS_SUCCESS)
+		STARPU_CUBLAS_REPORT_ERROR(status);
 }
 #endif
 
@@ -83,18 +83,32 @@ void axpy_gpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
 extern void axpy_opencl(void *buffers[], void *args);
 #endif
 
+static struct starpu_perfmodel axpy_model =
+{
+	.type = STARPU_HISTORY_BASED,
+	.symbol = "axpy"
+};
+
 static struct starpu_codelet axpy_cl =
 {
 	.cpu_funcs = {axpy_cpu},
+	.cpu_funcs_name = {"axpy_cpu"},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {axpy_gpu},
+#elif defined(STARPU_SIMGRID)
+	.cuda_funcs = {(void*)1},
 #endif
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {axpy_opencl},
+#elif defined(STARPU_SIMGRID)
+	.opencl_funcs = {(void*)1},
 #endif
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_RW},
-	.name = "axpy"
+	.name = "axpy",
+	.model = &axpy_model
 };
 
 static int
@@ -104,7 +118,8 @@ check(void)
 	for (i = 0; i < N; i++)
 	{
 		TYPE expected_value = _alpha * _vec_x[i] + 4.0;
-		if (fabs(_vec_y[i] - expected_value) > expected_value * EPSILON) {
+		if (fabs(_vec_y[i] - expected_value) > expected_value * EPSILON)
+		{
 			FPRINTF(stderr,"at %d, %f*%f+%f=%f, expected %f\n", i, _alpha, _vec_x[i], 4.0, _vec_y[i], expected_value);
 			return EXIT_FAILURE;
 		}
@@ -117,7 +132,7 @@ check(void)
 struct starpu_opencl_program opencl_program;
 #endif
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret, exit_value = 0;
 
@@ -156,8 +171,8 @@ int main(int argc, char **argv)
 	FPRINTF(stderr, "BEFORE y[0] = %2.2f\n", _vec_y[0]);
 
 	/* Declare the data to StarPU */
-	starpu_vector_data_register(&_handle_x, 0, (uintptr_t)_vec_x, N, sizeof(TYPE));
-	starpu_vector_data_register(&_handle_y, 0, (uintptr_t)_vec_y, N, sizeof(TYPE));
+	starpu_vector_data_register(&_handle_x, STARPU_MAIN_RAM, (uintptr_t)_vec_x, N, sizeof(TYPE));
+	starpu_vector_data_register(&_handle_y, STARPU_MAIN_RAM, (uintptr_t)_vec_y, N, sizeof(TYPE));
 
 	/* Divide the vector into blocks */
 	struct starpu_data_filter block_filter =
@@ -182,10 +197,13 @@ int main(int argc, char **argv)
 		task->cl = &axpy_cl;
 
 		task->cl_arg = &_alpha;
+		task->cl_arg_size = sizeof(_alpha);
 
 		task->handles[0] = starpu_data_get_sub_data(_handle_x, 1, b);
 		task->handles[1] = starpu_data_get_sub_data(_handle_y, 1, b);
 
+		task->tag_id = b;
+
 		ret = starpu_task_submit(task);
 		if (ret == -ENODEV)
 		{
@@ -198,8 +216,8 @@ int main(int argc, char **argv)
 	starpu_task_wait_for_all();
 
 enodev:
-	starpu_data_unpartition(_handle_x, 0);
-	starpu_data_unpartition(_handle_y, 0);
+	starpu_data_unpartition(_handle_x, STARPU_MAIN_RAM);
+	starpu_data_unpartition(_handle_y, STARPU_MAIN_RAM);
 	starpu_data_unregister(_handle_x);
 	starpu_data_unregister(_handle_y);
 
