@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2017                                CNRS
- * Copyright (C) 2009,2010,2014,2015,2017,2018            Université de Bordeaux
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2013                                     Inria
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -73,7 +71,7 @@ int exchange(int rank, starpu_data_handle_t *handles, check_func func, int detac
 		}
 
 		// We sleep to make sure that the data for the tag 9 will be received before the recv is posted
-		usleep(2000000);
+		starpu_sleep(2);
 		for(i=1 ; i<NB ; i++)
 		{
 			if (detached)
@@ -113,12 +111,14 @@ void check_variable(starpu_data_handle_t handle, int i, int rank, int *error)
 {
 	int other_rank = rank%2 == 0 ? rank+1 : rank-1;
 
+	starpu_data_acquire_on_node(handle, starpu_worker_get_local_memory_node(), STARPU_R);
 	int *rvalue = (int *)starpu_data_get_local_ptr(handle);
 	if (*rvalue != i*other_rank)
 	{
 		FPRINTF_MPI(stderr, "Incorrect received value: %d != %d\n", *rvalue, i*other_rank);
 		*error = 1;
 	}
+	starpu_data_release(handle);
 }
 
 int exchange_variable(int rank, int detached)
