@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2017                                CNRS
- * Copyright (C) 2016-2017                                Université de Bordeaux
+ * Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +28,7 @@
 #include <starpu.h>
 #include <core/jobs.h>
 #include <common/graph.h>
+#include <core/workers.h>
 
 /* Protects the whole task graph except the dropped list */
 static starpu_pthread_rwlock_t graph_lock;
@@ -37,39 +37,43 @@ static starpu_pthread_rwlock_t graph_lock;
 int _starpu_graph_record;
 
 /* This list contains all nodes without incoming dependency */
-struct _starpu_graph_node_multilist_top top;
+static struct _starpu_graph_node_multilist_top top;
 /* This list contains all nodes without outgoing dependency */
-struct _starpu_graph_node_multilist_bottom bottom;
+static struct _starpu_graph_node_multilist_bottom bottom;
 /* This list contains all nodes */
-struct _starpu_graph_node_multilist_all all;
+static struct _starpu_graph_node_multilist_all all;
 
 /* Protects the dropped list, always taken before graph lock */
 static starpu_pthread_mutex_t dropped_lock;
 /* This list contains all dropped nodes, i.e. the job terminated by the corresponding node is still int he graph */
-struct _starpu_graph_node_multilist_dropped dropped;
+static struct _starpu_graph_node_multilist_dropped dropped;
 
 void _starpu_graph_init(void)
 {
 	STARPU_PTHREAD_RWLOCK_INIT(&graph_lock, NULL);
-	_starpu_graph_node_multilist_init_top(&top);
-	_starpu_graph_node_multilist_init_bottom(&bottom);
-	_starpu_graph_node_multilist_init_all(&all);
+	_starpu_graph_node_multilist_head_init_top(&top);
+	_starpu_graph_node_multilist_head_init_bottom(&bottom);
+	_starpu_graph_node_multilist_head_init_all(&all);
 	STARPU_PTHREAD_MUTEX_INIT(&dropped_lock, NULL);
-	_starpu_graph_node_multilist_init_dropped(&dropped);
+	_starpu_graph_node_multilist_head_init_dropped(&dropped);
 }
 
+/* LockWR the graph lock */
 void _starpu_graph_wrlock(void)
 {
+	starpu_worker_relax_on();
 	STARPU_PTHREAD_RWLOCK_WRLOCK(&graph_lock);
+	starpu_worker_relax_off();
 }
 
 void _starpu_graph_drop_node(struct _starpu_graph_node *node);
 
+/* This flushes the list of nodes to be dropped. Both the dropped_lock and
+ * graph_lock mutexes have to be held on entry, and are released.  */
 void _starpu_graph_drop_dropped_nodes(void)
 {
 	struct _starpu_graph_node_multilist_dropped dropping;
 
-	STARPU_PTHREAD_MUTEX_LOCK(&dropped_lock);
 	/* Pick up the list of dropped nodes */
 	_starpu_graph_node_multilist_move_dropped(&dropped, &dropping);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&dropped_lock);
@@ -79,7 +83,6 @@ void _starpu_graph_drop_dropped_nodes(void)
 	{
 		struct _starpu_graph_node *node, *next;
 
-		STARPU_PTHREAD_RWLOCK_WRLOCK(&graph_lock);
 		for (node = _starpu_graph_node_multilist_begin_dropped(&dropping);
 		     node != _starpu_graph_node_multilist_end_dropped(&dropping);
 		     node = next)
@@ -87,24 +90,35 @@ void _starpu_graph_drop_dropped_nodes(void)
 			next = _starpu_graph_node_multilist_next_dropped(node);
 			_starpu_graph_drop_node(node);
 		}
-		STARPU_PTHREAD_RWLOCK_UNLOCK(&graph_lock);
 	}
+	STARPU_PTHREAD_RWLOCK_UNLOCK(&graph_lock);
 }
 
+/* UnlockWR the graph lock */
 void _starpu_graph_wrunlock(void)
 {
-	STARPU_PTHREAD_RWLOCK_UNLOCK(&graph_lock);
+	starpu_worker_relax_on();
+	STARPU_PTHREAD_MUTEX_LOCK(&dropped_lock);
+	starpu_worker_relax_off();
 	_starpu_graph_drop_dropped_nodes();
 }
 
+/* LockRD the graph lock */
 void _starpu_graph_rdlock(void)
 {
+	starpu_worker_relax_on();
 	STARPU_PTHREAD_RWLOCK_RDLOCK(&graph_lock);
+	starpu_worker_relax_off();
 }
 
+/* UnlockRD the graph lock */
 void _starpu_graph_rdunlock(void)
 {
 	STARPU_PTHREAD_RWLOCK_UNLOCK(&graph_lock);
+	/* Take the opportunity to try to take it WR */
+	if (STARPU_PTHREAD_RWLOCK_TRYWRLOCK(&graph_lock) == 0)
+		/* Good, flush dropped nodes */
+		_starpu_graph_wrunlock();
 }
 
 static void __starpu_graph_foreach(void (*func)(void *data, struct _starpu_graph_node *node), void *data)
@@ -148,7 +162,9 @@ static unsigned add_node(struct _starpu_graph_node *node, struct _starpu_graph_n
 			*alloc_nodes = 4;
 		_STARPU_REALLOC(*nodes, *alloc_nodes * sizeof(**nodes));
 		if (slot)
+		{
 			_STARPU_REALLOC(*slot, *alloc_nodes * sizeof(**slot));
+		}
 	}
 	ret = (*n_nodes)++;
 	(*nodes)[ret] = node;
@@ -238,22 +254,25 @@ void _starpu_graph_drop_job(struct _starpu_job *job)
 	if (!node)
 		return;
 
+	starpu_worker_relax_on();
 	STARPU_PTHREAD_MUTEX_LOCK(&node->mutex);
+	starpu_worker_relax_off();
 	/* Will not be able to use the job any more */
 	node->job = NULL;
 	STARPU_PTHREAD_MUTEX_UNLOCK(&node->mutex);
 
+	starpu_worker_relax_on();
 	STARPU_PTHREAD_MUTEX_LOCK(&dropped_lock);
+	starpu_worker_relax_off();
+	/* Queue for removal when lock becomes available */
+	_starpu_graph_node_multilist_push_back_dropped(&dropped, node);
 	if (STARPU_PTHREAD_RWLOCK_TRYWRLOCK(&graph_lock) == 0)
 	{
-		/* Graph wrlock is available, drop node immediately */
-		_starpu_graph_drop_node(node);
-		STARPU_PTHREAD_RWLOCK_UNLOCK(&graph_lock);
+		/* Graph wrlock is available, drop nodes immediately */
+		_starpu_graph_drop_dropped_nodes();
 	}
 	else
-		/* Queue for removal when lock becomes available */
-		_starpu_graph_node_multilist_push_back_dropped(&dropped, node);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&dropped_lock);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&dropped_lock);
 }
 
 static void _starpu_graph_set_n(void *data, struct _starpu_graph_node *node)
@@ -320,8 +339,9 @@ static void _starpu_graph_compute_bottom_up(void (*func)(struct _starpu_graph_no
 	free(next_set);
 }
 
-static void compute_depth(struct _starpu_graph_node *next_node, struct _starpu_graph_node *prev_node, void *data STARPU_ATTRIBUTE_UNUSED)
+static void compute_depth(struct _starpu_graph_node *next_node, struct _starpu_graph_node *prev_node, void *data)
 {
+	(void)data;
 	if (prev_node->depth < next_node->depth + 1)
 		prev_node->depth = next_node->depth + 1;
 }
