@@ -1,10 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016,2017                      Inria
- * Copyright (C) 2008-2019                                Université de Bordeaux
- * Copyright (C) 2010-2015,2017,2019                      CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +19,8 @@
 #ifndef __PERFMODEL_H__
 #define __PERFMODEL_H__
 
+/** @file */
+
 #include <common/config.h>
 #include <starpu.h>
 #include <core/task_bundle.h>
@@ -50,8 +50,10 @@ struct _starpu_perfmodel_state
 	starpu_pthread_rwlock_t model_rwlock;
 	int *nimpls;
 	int *nimpls_set;
-	int ncombs;  /* The number of combinations currently used by the model */
-	int ncombs_set; /* The number of combinations allocated in the array nimpls and ncombs */
+	/** The number of combinations currently used by the model */
+	int ncombs;
+	/** The number of combinations allocated in the array nimpls and ncombs */
+	int ncombs_set;
 	int *combs;
 };
 
@@ -105,6 +107,8 @@ void _starpu_simgrid_get_platform_path(int version, char *path, size_t maxlen);
 
 void _starpu_perfmodel_realloc(struct starpu_perfmodel *model, int nb);
 
+void _starpu_free_arch_combs(void);
+
 #if defined(STARPU_HAVE_HWLOC)
 hwloc_topology_t _starpu_perfmodel_get_hwtopology();
 #endif
