@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
@@ -115,7 +115,69 @@ static uint32_t _starpu_worker_exists_and_can_execute(struct starpu_task *task,
 	   task, independent of the sched_ctx, this latter may receive latter on
 	   the necessary worker - the user or the hypervisor should take care this happens */
 
-	struct _starpu_sched_ctx *sched_ctx = check_entire_platform == 1 ? _starpu_get_initial_sched_ctx() : _starpu_get_sched_ctx_struct(task->sched_ctx);
+	if (check_entire_platform && !task->cl->can_execute)
+	{
+		struct _starpu_machine_topology *topo = &_starpu_get_machine_config()->topology;
+
+		switch (arch)
+		{
+		case STARPU_CPU_WORKER:
+			if (!topo->ncpus)
+				return 0;
+			break;
+		case STARPU_CUDA_WORKER:
+			if (!topo->ncudagpus)
+				return 0;
+			break;
+		case STARPU_OPENCL_WORKER:
+			if (!topo->nopenclgpus)
+				return 0;
+			break;
+		case STARPU_MIC_WORKER:
+			if (!topo->nmicdevices)
+				return 0;
+			break;
+		case STARPU_MPI_MS_WORKER:
+			if (!topo->nmpidevices)
+				return 0;
+			break;
+		default:
+			STARPU_ABORT();
+		}
+
+		unsigned impl;
+		for (impl = 0; impl < STARPU_MAXIMPLEMENTATIONS; impl++)
+		{
+			switch (arch)
+			{
+			case STARPU_CPU_WORKER:
+				if (task->cl->cpu_funcs[impl] != NULL)
+					return 1;
+				break;
+			case STARPU_CUDA_WORKER:
+				if (task->cl->cuda_funcs[impl] != NULL)
+					return 1;
+				break;
+			case STARPU_OPENCL_WORKER:
+				if (task->cl->opencl_funcs[impl] != NULL)
+					return 1;
+				break;
+                        case STARPU_MIC_WORKER:
+                                if (task->cl->cpu_funcs_name[impl] != NULL || task->cl->mic_funcs[impl] != NULL)
+					return 1;
+                                break;
+                        case STARPU_MPI_MS_WORKER:
+                                if (task->cl->cpu_funcs_name[impl] != NULL || task->cl->mpi_ms_funcs[impl] != NULL)
+					return 1;
+                                break;
+			default:
+				STARPU_ABORT();
+			}
+		}
+		return 0;
+	}
+
+	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(task->sched_ctx);
 	struct starpu_worker_collection *workers = sched_ctx->workers;
 	struct starpu_sched_ctx_iterator it;
 	workers->init_iterator(workers, &it);
@@ -160,13 +222,14 @@ static uint32_t _starpu_worker_exists_and_can_execute(struct starpu_task *task,
 			}
 
 			if (!test_implementation)
+				/* No implementation here, cannot execute */
 				continue;
 
-			if (task->cl->can_execute)
-				return task->cl->can_execute(i, task, impl);
+			if (task->cl->can_execute && !task->cl->can_execute(i, task, impl))
+				/* The implementation cannot be executed here */
+				continue;
 
-			if(test_implementation)
-				return 1;
+			return 1;
 		}
 	}
 
@@ -1305,6 +1368,14 @@ int starpu_initialize(struct starpu_conf *user_conf, int *argc, char ***argv)
 		_STARPU_DISP("Warning: STARPU_ENABLE_STATS is enabled, which slows down a bit\n");
 	}
 
+#ifndef STARPU_SIMGRID
+	if (starpu_get_env_number_default("STARPU_SIMGRID", 0))
+	{
+		_STARPU_DISP("Simulation mode requested, but this libstarpu was built without simgrid support, please recompile\n");
+		return -EINVAL;
+	}
+#endif
+
 #if defined(_WIN32) && !defined(__CYGWIN__)
 	WSADATA wsadata;
 	WSAStartup(MAKEWORD(1,0), &wsadata);
@@ -1449,6 +1520,15 @@ int starpu_initialize(struct starpu_conf *user_conf, int *argc, char ***argv)
 	STARPU_PTHREAD_MUTEX_UNLOCK(&init_mutex);
 
 	int main_thread_cpuid = starpu_get_env_number_default("STARPU_MAIN_THREAD_CPUID", -1);
+	int main_thread_coreid = starpu_get_env_number_default("STARPU_MAIN_THREAD_COREID", -1);
+	if (main_thread_cpuid >= 0 && main_thread_coreid >= 0)
+	{
+		_STARPU_DISP("Warning: STARPU_MAIN_THREAD_CPUID and STARPU_MAIN_THREAD_COREID cannot be set at the same time. STARPU_MAIN_THREAD_CPUID will be used.\n");
+	}
+	if (main_thread_cpuid == -1 && main_thread_coreid >= 0)
+	{
+		main_thread_cpuid = main_thread_coreid * _starpu_get_nhyperthreads();
+	}
 	int main_thread_bind = starpu_get_env_number_default("STARPU_MAIN_THREAD_BIND", 0);
 	int main_thread_activity = STARPU_NONACTIVETHREAD;
 	if (main_thread_bind)
