@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016,2017                      Inria
- * Copyright (C) 2018                                     Federal University of Rio Grande do Sul (UFRGS)
- * Copyright (C) 2010-2017, 2019                          CNRS
- * Copyright (C) 2009-2019                                Université de Bordeaux
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2018       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -61,12 +59,9 @@ static int malloc_on_node_default_flags[STARPU_MAXNODES];
 
 /* This file is used for implementing "folded" allocation */
 #ifdef STARPU_SIMGRID
-#if SIMGRID_VERSION < 31500 || SIMGRID_VERSION == 31559
-/* TODO: drop when simgrid 3.15 is reasonably largely used by people who need the feature */
 static int bogusfile = -1;
 static unsigned long _starpu_malloc_simulation_fold;
 #endif
-#endif
 
 static starpu_malloc_hook malloc_hook;
 static starpu_free_hook free_hook;
@@ -154,6 +149,15 @@ static int _starpu_malloc_should_pin(int flags)
 	return 0;
 }
 
+int _starpu_malloc_willpin_on_node(unsigned dst_node)
+{
+	int flags = malloc_on_node_default_flags[dst_node];
+	return (_starpu_malloc_should_pin(flags) && STARPU_RUNNING_ON_VALGRIND == 0
+			&& (_starpu_can_submit_cuda_task()
+			    /* || _starpu_can_submit_opencl_task() */
+			));
+}
+
 int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int flags)
 {
 	int ret=0;
@@ -190,6 +194,7 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 		goto end;
 	}
 
+	/* Note: synchronize this test with _starpu_malloc_willpin_on_node */
 	if (_starpu_malloc_should_pin(flags) && STARPU_RUNNING_ON_VALGRIND == 0)
 	{
 		if (_starpu_can_submit_cuda_task())
@@ -200,7 +205,7 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 		 * instead of computing an average value.
 		 */
 			if (_starpu_simgrid_cuda_malloc_cost())
-				MSG_process_sleep((float) dim * 0.000650 / 1048576.);
+				starpu_sleep((float) dim * 0.000650 / 1048576.);
 #else /* STARPU_SIMGRID */
 #ifdef STARPU_USE_CUDA
 #ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
@@ -279,9 +284,11 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
 	{
 #if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
-		*A = SMPI_SHARED_MALLOC(dim);
-#else
-		/* TODO: drop when simgrid 3.15 is reasonably largely used by people who need the feature */
+	    if (_starpu_simgrid_running_smpi())
+		    *A = SMPI_SHARED_MALLOC(dim);
+	    else
+#endif
+	    {
 		/* Use "folded" allocation: the same file is mapped several
 		 * times contiguously, to get a memory area one can read/write,
 		 * without consuming memory */
@@ -338,7 +345,7 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 			}
 			*A = buf;
 		}
-#endif
+	    }
 	}
 #endif
 #ifdef STARPU_HAVE_HWLOC
@@ -527,11 +534,11 @@ int _starpu_free_flags_on_node(unsigned dst_node, void *A, size_t dim, int flags
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
 	{
 #if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
+	    if (_starpu_simgrid_running_smpi())
 		SMPI_SHARED_FREE(A);
-#else
-		/* TODO: drop when simgrid 3.15 is reasonably largely used by people who need the feature */
-		munmap(A, dim);
+	    else
 #endif
+		munmap(A, dim);
 	}
 #endif
 #ifdef STARPU_HAVE_HWLOC
@@ -574,7 +581,7 @@ static uintptr_t _starpu_malloc_on_node(unsigned dst_node, size_t size, int flag
 
 	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
 	if (node_ops && node_ops->malloc_on_node)
-		return node_ops->malloc_on_node(dst_node, size, flags);
+		addr = node_ops->malloc_on_node(dst_node, size, flags & ~STARPU_MALLOC_COUNT);
 	else
 		STARPU_ABORT_MSG("No malloc_on_node function defined for node %s\n", _starpu_node_get_prefix(starpu_node_get_kind(dst_node)));
 
@@ -582,7 +589,8 @@ static uintptr_t _starpu_malloc_on_node(unsigned dst_node, size_t size, int flag
 	{
 		// Allocation failed, gives the memory back to the memory manager
 		_STARPU_TRACE_MEMORY_FULL(size);
-		starpu_memory_deallocate(dst_node, size);
+		if (flags & STARPU_MALLOC_COUNT)
+			starpu_memory_deallocate(dst_node, size);
 	}
 	return addr;
 }
@@ -695,11 +703,9 @@ _starpu_malloc_init(unsigned dst_node)
 	disable_pinning = starpu_get_env_number("STARPU_DISABLE_PINNING");
 	malloc_on_node_default_flags[dst_node] = STARPU_MALLOC_PINNED | STARPU_MALLOC_COUNT;
 #ifdef STARPU_SIMGRID
-#if SIMGRID_VERSION < 31500 || SIMGRID_VERSION == 31559
 	/* Reasonably "costless" */
 	_starpu_malloc_simulation_fold = starpu_get_env_number_default("STARPU_MALLOC_SIMULATION_FOLD", 1) << 20;
 #endif
-#endif
 }
 
 void
