@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2015                                Université de Bordeaux
- * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +17,8 @@
 #ifndef __MEMALLOC_H__
 #define __MEMALLOC_H__
 
+/** @file */
+
 #include <starpu.h>
 #include <common/config.h>
 
@@ -25,13 +26,14 @@
 #include <datawizard/interfaces/data_interface.h>
 #include <datawizard/coherency.h>
 #include <datawizard/copy_driver.h>
+#include <datawizard/data_request.h>
 
 struct _starpu_data_replicate;
 
-/* While associated with a handle, the content is protected by the handle lock, except a few fields
+/** While associated with a handle, the content is protected by the handle lock, except a few fields
  */
 LIST_TYPE(_starpu_mem_chunk,
-	/* protected by the mc_lock */
+	/** protected by the mc_lock */
 	starpu_data_handle_t data;
 
 	uint32_t footprint;
@@ -48,17 +50,21 @@ LIST_TYPE(_starpu_mem_chunk,
 	void *chunk_interface;
 	size_t size_interface;
 
-	/* Whether StarPU automatically allocated this memory, or the application did */
+	/** Whether StarPU automatically allocated this memory, or the application did */
 	unsigned automatically_allocated:1;
-	/* A buffer that is used for SCRATCH or reduction cannnot be used with
+	/** A buffer that is used for SCRATCH or reduction cannnot be used with
 	 * filters. */
 	unsigned relaxed_coherency:2;
-	/* Whether this is the home chunk, or there is no home chunk (and it is thus always clean) */
+	/** Whether this is the home chunk, or there is no home chunk (and it is thus always clean) */
 	unsigned home:1;
-	/* Whether the memchunk is in the clean part of the mc_list */
+	/** Whether the memchunk is in the clean part of the mc_list */
 	unsigned clean:1;
+	/** Was this chunk used since it got allocated?  */
+	unsigned diduse:1;
+	/** Was this chunk marked as "won't use"? */
+	unsigned wontuse:1;
 
-	/* the size of the data is only set when calling _starpu_request_mem_chunk_removal(),
+	/** the size of the data is only set when calling _starpu_request_mem_chunk_removal(),
 	 * it is needed to estimate how much memory is in mc_cache, and by
 	 * free_memory_on_node() which is called when the handle is no longer
 	 * valid.
@@ -68,7 +74,7 @@ LIST_TYPE(_starpu_mem_chunk,
 
 	struct _starpu_data_replicate *replicate;
 
-	/* This is set when one keeps a pointer to this mc obtained from the
+	/** This is set when one keeps a pointer to this mc obtained from the
 	 * mc_list without mc_lock held. We need to clear the pointer if we
 	 * remove this entry from the mc_list, so we know we have to restart
 	 * from zero. This is protected by the corresponding mc_lock.  */
@@ -77,8 +83,9 @@ LIST_TYPE(_starpu_mem_chunk,
 
 void _starpu_init_mem_chunk_lists(void);
 void _starpu_deinit_mem_chunk_lists(void);
+void _starpu_mem_chunk_init_last(void);
 void _starpu_request_mem_chunk_removal(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate, unsigned node, size_t size);
-int _starpu_allocate_memory_on_node(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate, unsigned is_prefetch);
+int _starpu_allocate_memory_on_node(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate, enum _starpu_is_prefetch is_prefetch);
 size_t _starpu_free_all_automatically_allocated_buffers(unsigned node);
 void _starpu_memchunk_recently_used(struct _starpu_mem_chunk *mc, unsigned node);
 void _starpu_memchunk_wont_use(struct _starpu_mem_chunk *m, unsigned nodec);
@@ -88,4 +95,6 @@ void _starpu_display_memory_stats_by_node(int node);
 size_t _starpu_memory_reclaim_generic(unsigned node, unsigned force, size_t reclaim);
 int _starpu_is_reclaiming(unsigned node);
 
+void _starpu_mem_chunk_disk_register(unsigned disk_memnode);
+
 #endif
