@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013,2015-2017                           CNRS
- * Copyright (C) 2013-2016                                Université de Bordeaux
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2012-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -37,7 +35,7 @@
 
 /* Array of structures containing all the informations useful to send
  * and receive informations with devices */
-struct _starpu_mp_node *mic_nodes[STARPU_MAXMICDEVS];
+struct _starpu_mp_node *_starpu_mic_nodes[STARPU_MAXMICDEVS];
 
 static COIENGINE handles[STARPU_MAXMICDEVS];
 
@@ -85,15 +83,15 @@ struct _starpu_mp_node *_starpu_mic_src_get_actual_thread_mp_node()
 	int devid = actual_worker->devid;
 	STARPU_ASSERT(devid >= 0 && devid < STARPU_MAXMICDEVS);
 
-	return mic_nodes[devid];
+	return _starpu_mic_nodes[devid];
 }
 
-const struct _starpu_mp_node *_starpu_mic_src_get_mp_node_from_memory_node(int memory_node)
+struct _starpu_mp_node *_starpu_mic_src_get_mp_node_from_memory_node(int memory_node)
 {
-	int devid = _starpu_memory_node_get_devid(memory_node);
+	int devid = starpu_memory_node_get_devid(memory_node);
 	STARPU_ASSERT_MSG(devid >= 0 && devid < STARPU_MAXMICDEVS, "bogus devid %d for memory node %d\n", devid, memory_node);
 
-	return mic_nodes[devid];
+	return _starpu_mic_nodes[devid];
 }
 
 static void _starpu_mic_src_free_kernel(void *kernel)
@@ -106,7 +104,7 @@ static void _starpu_mic_src_free_kernel(void *kernel)
 
 void _starpu_mic_clear_kernels(void)
 {
-	struct _starpu_mic_kernel *kernel, *tmp;
+	struct _starpu_mic_kernel *kernel=NULL, *tmp=NULL;
 	HASH_ITER(hh, kernels, kernel, tmp)
 	{
 		HASH_DEL(kernels, kernel);
@@ -178,7 +176,7 @@ starpu_mic_kernel_t _starpu_mic_src_get_kernel(starpu_mic_func_symbol_t symbol)
 
 	if (kernel->func[devid] == NULL)
 	{
-		struct _starpu_mp_node *node = mic_nodes[devid];
+		struct _starpu_mp_node *node = _starpu_mic_nodes[devid];
 		int ret = _starpu_src_common_lookup(node, (void (**)(void))&kernel->func[devid], kernel->name);
 		if (ret)
 			return NULL;
@@ -196,8 +194,7 @@ void _starpu_mic_src_report_coi_error(const char *func, const char *file,
 				      const int line, const COIRESULT status)
 {
 	const char *errormsg = COIResultGetName(status);
-	printf("SRC: oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
-	STARPU_ASSERT(0);
+	_STARPU_ERROR("SRC: oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
 }
 
 /* Report an error which occured when using a MIC device
@@ -208,8 +205,7 @@ void _starpu_mic_src_report_coi_error(const char *func, const char *file,
 void _starpu_mic_src_report_scif_error(const char *func, const char *file, const int line, const int status)
 {
 	const char *errormsg = strerror(status);
-	printf("SRC: oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
-	STARPU_ASSERT(0);
+	_STARPU_ERROR("SRC: oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
 }
 
 /* Return the number of MIC devices in the system.
@@ -266,7 +262,7 @@ starpu_mic_kernel_t _starpu_mic_src_get_kernel_from_codelet(struct starpu_codele
 		/* If user dont define any starpu_mic_fun_t in cl->mic_func we try to use
 		 * cpu_func_name.
 		 */
-		char *func_name = _starpu_task_get_cpu_name_nth_implementation(cl, nimpl);
+		const char *func_name = _starpu_task_get_cpu_name_nth_implementation(cl, nimpl);
 		if (func_name)
 		{
 			starpu_mic_func_symbol_t symbol;
@@ -301,7 +297,7 @@ void(* _starpu_mic_src_get_kernel_from_job(const struct _starpu_mp_node *node ST
 		/* If user dont define any starpu_mic_fun_t in cl->mic_func we try to use
 		 * cpu_func_name.
 		 */
-		char *func_name = _starpu_task_get_cpu_name_nth_implementation(j->task->cl, j->nimpl);
+		const char *func_name = _starpu_task_get_cpu_name_nth_implementation(j->task->cl, j->nimpl);
 		if (func_name)
 		{
 			starpu_mic_func_symbol_t symbol;
@@ -383,7 +379,7 @@ int _starpu_mic_allocate_memory(void **addr, size_t size, unsigned memory_node)
 	 * transfert with scif is not possible when the MIC
 	 * doesn't have enought free memory.
 	 * In this cas we can't tell any things to the host. */
-	//int devid = _starpu_memory_node_get_devid(memory_node);
+	//int devid = starpu_memory_node_get_devid(memory_node);
 	//if (_starpu_mic_get_free_mem_size(devid) < size * 1.25)
 	//	return 1;
 
@@ -400,7 +396,7 @@ void _starpu_mic_free_memory(void *addr, size_t size, unsigned memory_node)
 	const struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(memory_node);
 	struct _starpu_mic_free_command cmd = {addr, size};
 
-	return _starpu_mp_common_send_command(mp_node, STARPU_FREE, &cmd, sizeof(cmd));
+	return _starpu_mp_common_send_command(mp_node, STARPU_MP_COMMAND_FREE, &cmd, sizeof(cmd));
 }
 
 /* Transfert SIZE bytes from the address pointed by SRC in the SRC_NODE memory
@@ -408,9 +404,9 @@ void _starpu_mic_free_memory(void *addr, size_t size, unsigned memory_node)
  */
 int _starpu_mic_copy_ram_to_mic(void *src, unsigned src_node STARPU_ATTRIBUTE_UNUSED, void *dst, unsigned dst_node, size_t size)
 {
-	const struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(dst_node);
+	struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(dst_node);
 
-	return _starpu_src_common_copy_host_to_sink(mp_node, src, dst, size);
+	return _starpu_src_common_copy_host_to_sink_sync(mp_node, src, dst, size);
 }
 
 /* Transfert SIZE bytes from the address pointed by SRC in the SRC_NODE memory
@@ -418,9 +414,9 @@ int _starpu_mic_copy_ram_to_mic(void *src, unsigned src_node STARPU_ATTRIBUTE_UN
  */
 int _starpu_mic_copy_mic_to_ram(void *src, unsigned src_node, void *dst, unsigned dst_node STARPU_ATTRIBUTE_UNUSED, size_t size)
 {
-	const struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(src_node);
+	struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(src_node);
 
-	return _starpu_src_common_copy_sink_to_host(mp_node, src, dst, size);
+	return _starpu_src_common_copy_sink_to_host_sync(mp_node, src, dst, size);
 }
 
 /* Asynchronous transfers */
@@ -473,9 +469,29 @@ int _starpu_mic_init_event(struct _starpu_mic_async_event *event, unsigned memor
 	return 0;
 }
 
-/* Wait the end of the asynchronous request */
-void _starpu_mic_wait_request_completion(struct _starpu_mic_async_event *event)
+/* Test if a asynchronous request is end.
+ * Return 1 if is end, 0 else. */
+unsigned _starpu_mic_test_request_completion(struct _starpu_async_channel *async_channel)
 {
+	struct _starpu_mic_async_event *event = &async_channel->event.mic_event;
+	if (event->signal != NULL && *(event->signal) != STARPU_MIC_REQUEST_COMPLETE)
+		return 0;
+
+	const struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(event->memory_node);
+	scif_epd_t epd = mp_node->host_sink_dt_connection.mic_endpoint;
+
+	if (scif_unregister(epd, (off_t)(event->signal), 0x1000) < 0)
+		STARPU_MIC_SRC_REPORT_SCIF_ERROR(errno);
+
+	free(event->signal);
+	event->signal = NULL;
+	return 1;
+}
+
+/* Wait the end of the asynchronous request */
+void _starpu_mic_wait_request_completion(struct _starpu_async_channel *async_channel)
+{
+	struct _starpu_mic_async_event *event = &async_channel->event.mic_event;
 	if (event->signal != NULL)
 	{
 		const struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(event->memory_node);
@@ -492,26 +508,6 @@ void _starpu_mic_wait_request_completion(struct _starpu_mic_async_event *event)
 	}
 }
 
-/* Test if a asynchronous request is end.
- * Return 1 if is end, 0 else. */
-int _starpu_mic_request_is_complete(struct _starpu_mic_async_event *event)
-{
-	if (event->signal != NULL && *(event->signal) != STARPU_MIC_REQUEST_COMPLETE)
-		return 0;
-
-	const struct _starpu_mp_node *mp_node = _starpu_mic_src_get_mp_node_from_memory_node(event->memory_node);
-	scif_epd_t epd = mp_node->host_sink_dt_connection.mic_endpoint;
-
-	if (scif_unregister(epd, (off_t)(event->signal), 0x1000) < 0)
-		STARPU_MIC_SRC_REPORT_SCIF_ERROR(errno);
-
-	free(event->signal);
-	event->signal = NULL;
-	return 1;
-}
-
-
-
 void *_starpu_mic_src_worker(void *arg)
 {
 	struct _starpu_worker_set *worker_set = arg;
@@ -559,8 +555,159 @@ void *_starpu_mic_src_worker(void *arg)
 	STARPU_PTHREAD_COND_SIGNAL(&worker_set->ready_cond);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&worker_set->mutex);
 
-	_starpu_src_common_worker(worker_set, baseworkerid, mic_nodes[devid]);
+	_starpu_src_common_worker(worker_set, baseworkerid, _starpu_mic_nodes[devid]);
 
 	return NULL;
 
 }
+
+int _starpu_mic_copy_interface_from_mic_to_cpu(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+{
+	int src_kind = starpu_node_get_kind(src_node);
+	int dst_kind = starpu_node_get_kind(dst_node);
+	STARPU_ASSERT(src_kind == STARPU_MIC_RAM && dst_kind == STARPU_CPU_RAM);
+
+	int ret = 0;
+	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
+	/* MIC -> RAM */
+	if (!req || starpu_asynchronous_copy_disabled() || starpu_asynchronous_mic_copy_disabled() || !(copy_methods->mic_to_ram_async || copy_methods->any_to_any))
+	{
+		/* this is not associated to a request so it's synchronous */
+		STARPU_ASSERT(copy_methods->mic_to_ram || copy_methods->any_to_any);
+		if (copy_methods->mic_to_ram)
+			ret = copy_methods->mic_to_ram(src_interface, src_node, dst_interface, dst_node);
+		else
+			ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, NULL);
+	}
+	else
+	{
+		req->async_channel.node_ops = &_starpu_driver_mic_node_ops;
+		if (copy_methods->mic_to_ram_async)
+			ret = copy_methods->mic_to_ram_async(src_interface, src_node, dst_interface, dst_node);
+		else
+		{
+			STARPU_ASSERT(copy_methods->any_to_any);
+			ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, &req->async_channel);
+		}
+		_starpu_mic_init_event(&(req->async_channel.event.mic_event), src_node);
+	}
+	return ret;
+}
+
+int _starpu_mic_copy_interface_from_cpu_to_mic(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+{
+	int src_kind = starpu_node_get_kind(src_node);
+	int dst_kind = starpu_node_get_kind(dst_node);
+	STARPU_ASSERT(src_kind == STARPU_CPU_RAM && dst_kind == STARPU_MIC_RAM);
+
+	int ret = 0;
+	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
+	/* RAM -> MIC */
+	if (!req || starpu_asynchronous_copy_disabled() || starpu_asynchronous_mic_copy_disabled() || !(copy_methods->ram_to_mic_async || copy_methods->any_to_any))
+	{
+		/* this is not associated to a request so it's synchronous */
+		STARPU_ASSERT(copy_methods->ram_to_mic || copy_methods->any_to_any);
+		if (copy_methods->ram_to_mic)
+			ret = copy_methods->ram_to_mic(src_interface, src_node, dst_interface, dst_node);
+		else
+			ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, NULL);
+	}
+	else
+	{
+		req->async_channel.node_ops = &_starpu_driver_mic_node_ops;
+		if (copy_methods->ram_to_mic_async)
+			ret = copy_methods->ram_to_mic_async(src_interface, src_node, dst_interface, dst_node);
+		else
+		{
+			STARPU_ASSERT(copy_methods->any_to_any);
+			ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, &req->async_channel);
+		}
+		_starpu_mic_init_event(&(req->async_channel.event.mic_event), dst_node);
+	}
+
+	return ret;
+}
+
+int _starpu_mic_copy_data_from_mic_to_cpu(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+{
+	int src_kind = starpu_node_get_kind(src_node);
+	int dst_kind = starpu_node_get_kind(dst_node);
+	STARPU_ASSERT(src_kind == STARPU_MIC_RAM && dst_kind == STARPU_CPU_RAM);
+
+	if (async_channel)
+		return _starpu_mic_copy_mic_to_ram_async((void*) (src + src_offset), src_node,
+							 (void*) (dst + dst_offset), dst_node,
+							 size);
+	else
+		return _starpu_mic_copy_mic_to_ram((void*) (src + src_offset), src_node,
+						   (void*) (dst + dst_offset), dst_node,
+						   size);
+}
+
+int _starpu_mic_copy_data_from_cpu_to_mic(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+{
+	int src_kind = starpu_node_get_kind(src_node);
+	int dst_kind = starpu_node_get_kind(dst_node);
+	STARPU_ASSERT(src_kind == STARPU_CPU_RAM && dst_kind == STARPU_MIC_RAM);
+
+	if (async_channel)
+		return _starpu_mic_copy_ram_to_mic_async((void*) (src + src_offset), src_node,
+							 (void*) (dst + dst_offset), dst_node,
+							 size);
+	else
+		return _starpu_mic_copy_ram_to_mic((void*) (src + src_offset), src_node,
+						   (void*) (dst + dst_offset), dst_node,
+						   size);
+}
+
+int _starpu_mic_is_direct_access_supported(unsigned node, unsigned handling_node)
+{
+	(void) node;
+	(void) handling_node;
+	/* TODO: We don't handle direct MIC-MIC transfers yet */
+	return 0;
+}
+
+uintptr_t _starpu_mic_malloc_on_node(unsigned dst_node, size_t size, int flags)
+{
+	(void) flags;
+	uintptr_t addr = 0;
+	if (_starpu_mic_allocate_memory((void **)(&addr), size, dst_node))
+		addr = 0;
+	return addr;
+}
+
+void _starpu_mic_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
+{
+	(void) flags;
+	_starpu_mic_free_memory((void*) addr, size, dst_node);
+}
+
+/* TODO: MIC -> MIC */
+struct _starpu_node_ops _starpu_driver_mic_node_ops =
+{
+	.copy_interface_to[STARPU_UNUSED] = NULL,
+	.copy_interface_to[STARPU_CPU_RAM] = _starpu_mic_copy_interface_from_mic_to_cpu,
+	.copy_interface_to[STARPU_CUDA_RAM] = NULL,
+	.copy_interface_to[STARPU_OPENCL_RAM] = NULL,
+	.copy_interface_to[STARPU_DISK_RAM] = NULL,
+	.copy_interface_to[STARPU_MIC_RAM] = NULL,
+	.copy_interface_to[STARPU_MPI_MS_RAM] = NULL,
+
+	.copy_data_to[STARPU_UNUSED] = NULL,
+	.copy_data_to[STARPU_CPU_RAM] = _starpu_mic_copy_data_from_mic_to_cpu,
+	.copy_data_to[STARPU_CUDA_RAM] = NULL,
+	.copy_data_to[STARPU_OPENCL_RAM] = NULL,
+	.copy_data_to[STARPU_DISK_RAM] = NULL,
+	.copy_data_to[STARPU_MIC_RAM] = NULL,
+	.copy_data_to[STARPU_MPI_MS_RAM] = NULL,
+
+	/* TODO: copy2D/3D? */
+
+	.wait_request_completion = _starpu_mic_wait_request_completion,
+	.test_request_completion = _starpu_mic_test_request_completion,
+	.is_direct_access_supported = _starpu_mic_is_direct_access_supported,
+	.malloc_on_node = _starpu_mic_malloc_on_node,
+	.free_on_node = _starpu_mic_free_on_node,
+	.name = "mic driver"
+};
