@@ -1,10 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2013,2015,2017,2018                 Inria
- * Copyright (C) 2011-2014,2016-2019                      CNRS
- * Copyright (C) 2011-2016,2018-2019                      Université de Bordeaux
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -32,7 +30,7 @@ struct _starpu_peager_common_data
 	int ref_count;
 };
 
-struct _starpu_peager_common_data *_peager_common_data = NULL;
+static struct _starpu_peager_common_data *_peager_common_data = NULL;
 
 struct _starpu_peager_data
 {
@@ -164,6 +162,9 @@ static void initialize_peager_policy(unsigned sched_ctx_id)
 {
 	struct _starpu_peager_data *data;
 	_STARPU_CALLOC(data, 1, sizeof(struct _starpu_peager_data));
+
+	_STARPU_DISP("Warning: the peager scheduler is mostly a proof of concept and not really very optimized\n");
+
 	/* masters pick tasks from that queue */
 	data->fifo = _starpu_create_fifo();
 
