@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,8 +36,8 @@
 #define _STARPU_INITIAL_THREAD_STACKSIZE 2097152
 
 static struct starpu_omp_global _global_state;
-starpu_pthread_key_t omp_thread_key;
-starpu_pthread_key_t omp_task_key;
+starpu_pthread_key_t _starpu_omp_thread_key;
+starpu_pthread_key_t _starpu_omp_task_key;
 
 struct starpu_omp_global *_starpu_omp_global_state = NULL;
 double _starpu_omp_clock_ref = 0.0; /* clock reference for starpu_omp_get_wtick */
@@ -57,26 +57,26 @@ static void wake_up_and_unlock_task(struct starpu_omp_task *task);
 static void wake_up_barrier(struct starpu_omp_region *parallel_region);
 static void starpu_omp_task_preempt(void);
 
-struct starpu_omp_thread *_starpu_omp_get_thread(void)
+struct starpu_omp_thread * _starpu_omp_get_thread(void)
 {
-	struct starpu_omp_thread *thread = STARPU_PTHREAD_GETSPECIFIC(omp_thread_key);
+	struct starpu_omp_thread *thread = STARPU_PTHREAD_GETSPECIFIC(_starpu_omp_thread_key);
 	return thread;
 }
 
 static inline void _starpu_omp_set_thread(struct starpu_omp_thread *thread)
 {
-	STARPU_PTHREAD_SETSPECIFIC(omp_thread_key, thread);
+	STARPU_PTHREAD_SETSPECIFIC(_starpu_omp_thread_key, thread);
 }
 
 struct starpu_omp_task *_starpu_omp_get_task(void)
 {
-	struct starpu_omp_task *task = STARPU_PTHREAD_GETSPECIFIC(omp_task_key);
+	struct starpu_omp_task *task = STARPU_PTHREAD_GETSPECIFIC(_starpu_omp_task_key);
 	return task;
 }
 
 static inline void _starpu_omp_set_task(struct starpu_omp_task *task)
 {
-	STARPU_PTHREAD_SETSPECIFIC(omp_task_key, task);
+	STARPU_PTHREAD_SETSPECIFIC(_starpu_omp_task_key, task);
 }
 
 struct starpu_omp_region *_starpu_omp_get_region_at_level(int level)
@@ -261,6 +261,11 @@ static struct starpu_omp_thread *get_local_thread(void)
 	return thread;
 }
 
+static struct starpu_omp_thread * __attribute__ ((noinline)) _get_local_thread_noinline(void)
+{
+	return get_local_thread();
+}
+
 static struct starpu_omp_critical *create_omp_critical_struct(void)
 {
 	struct starpu_omp_critical *critical;
@@ -467,7 +472,11 @@ static void starpu_omp_implicit_task_exec(void *buffers[], void *cl_arg)
 	struct starpu_omp_task *task = starpu_task_get_current()->omp_task;
 	STARPU_ASSERT(task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT);
 	_starpu_omp_set_task(task);
-	struct starpu_omp_thread *thread = get_local_thread();
+
+	/* get_local_thread() inlining triggers a clobbering warning with some
+	 * versions of GCC, thus we explicitly call the noinline variant */
+	struct starpu_omp_thread *thread = _get_local_thread_noinline();
+
 	if (task->state != starpu_omp_task_state_preempted)
 	{
 		task->starpu_buffers = buffers;
@@ -598,7 +607,10 @@ static void starpu_omp_explicit_task_exec(void *buffers[], void *cl_arg)
 	STARPU_ASSERT(!(task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT));
 	_starpu_omp_set_task(task);
 
-	struct starpu_omp_thread *thread = get_local_thread();
+	/* get_local_thread() inlining triggers a clobbering warning with some
+	 * versions of GCC, thus we explicitly call the noinline variant */
+	struct starpu_omp_thread *thread = _get_local_thread_noinline();
+
 	if (task->state != starpu_omp_task_state_preempted)
 	{
 		if (thread == NULL)
@@ -914,8 +926,8 @@ void _starpu_omp_dummy_init(void)
 {
 	if (_starpu_omp_global_state != &_global_state)
 	{
-		STARPU_PTHREAD_KEY_CREATE(&omp_thread_key, NULL);
-		STARPU_PTHREAD_KEY_CREATE(&omp_task_key, NULL);
+		STARPU_PTHREAD_KEY_CREATE(&_starpu_omp_thread_key, NULL);
+		STARPU_PTHREAD_KEY_CREATE(&_starpu_omp_task_key, NULL);
 	}
 }
 
@@ -926,8 +938,8 @@ void _starpu_omp_dummy_shutdown(void)
 {
 	if (_starpu_omp_global_state != &_global_state)
 	{
-		STARPU_PTHREAD_KEY_DELETE(omp_thread_key);
-		STARPU_PTHREAD_KEY_DELETE(omp_task_key);
+		STARPU_PTHREAD_KEY_DELETE(_starpu_omp_thread_key);
+		STARPU_PTHREAD_KEY_DELETE(_starpu_omp_task_key);
 	}
 }
 
@@ -952,8 +964,8 @@ int starpu_omp_init(void)
 
 	_starpu_omp_global_state = &_global_state;
 
-	STARPU_PTHREAD_KEY_CREATE(&omp_thread_key, NULL);
-	STARPU_PTHREAD_KEY_CREATE(&omp_task_key, NULL);
+	STARPU_PTHREAD_KEY_CREATE(&_starpu_omp_thread_key, NULL);
+	STARPU_PTHREAD_KEY_CREATE(&_starpu_omp_task_key, NULL);
 	_global_state.initial_device = create_omp_device_struct();
 	_global_state.initial_region = create_omp_region_struct(NULL, _global_state.initial_device);
 	_global_state.initial_thread = create_omp_thread_struct(_global_state.initial_region);
@@ -1021,8 +1033,8 @@ void starpu_omp_shutdown(void)
 	_starpu_spin_unlock(&_global_state.hash_workers_lock);
 	_starpu_spin_destroy(&_global_state.hash_workers_lock);
 	_starpu_omp_environment_exit();
-	STARPU_PTHREAD_KEY_DELETE(omp_task_key);
-	STARPU_PTHREAD_KEY_DELETE(omp_thread_key);
+	STARPU_PTHREAD_KEY_DELETE(_starpu_omp_task_key);
+	STARPU_PTHREAD_KEY_DELETE(_starpu_omp_thread_key);
 #ifdef STARPU_SIMGRID
 	_starpu_simgrid_deinit_late();
 #endif
