@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -79,7 +79,7 @@ static void common_data_cpy_func(void *descr[], void *cl_arg)
 
 }
 
-void mp_cpy_kernel(void *descr[], void *cl_arg)
+static void mp_cpy_kernel(void *descr[], void *cl_arg)
 {
 	unsigned interface_id = *(unsigned *)cl_arg;
 
@@ -111,7 +111,7 @@ static starpu_mic_kernel_t mic_cpy_func()
 #endif
 }
 
-struct starpu_perfmodel copy_model =
+static struct starpu_perfmodel copy_model =
 {
 	.type = STARPU_HISTORY_BASED,
 	.symbol = "starpu_data_cpy"
