@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2014, 2016  Université de Bordeaux
- * Copyright (C) 2012-2013, 2016, 2017  CNRS
- * Copyright (C) 2011-2013  INRIA
+ * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,10 +15,11 @@
  */
 
 #include <starpu.h>
+#include "core/workers.h"
 
-static unsigned list_has_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+static unsigned list_has_next_unblocked_worker(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
 {
-	int nworkers = workers->nworkers;
+	int nworkers = workers->nunblocked_workers;
 	STARPU_ASSERT(it != NULL);
 
 	unsigned ret = it->cursor < nworkers ;
@@ -30,10 +29,10 @@ static unsigned list_has_next(struct starpu_worker_collection *workers, struct s
 	return ret;
 }
 
-static int list_get_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+static int list_get_next_unblocked_worker(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
 {
-	int *workerids = (int *)workers->workerids;
-	int nworkers = (int)workers->nworkers;
+	int *workerids = (int *)workers->unblocked_workers;
+	int nworkers = (int)workers->nunblocked_workers;
 
 	STARPU_ASSERT(it->cursor < nworkers);
 
@@ -66,6 +65,40 @@ static int list_get_next_master(struct starpu_worker_collection *workers, struct
 	return ret;
 }
 
+static unsigned list_has_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+{
+	if(it->possibly_parallel == 1)
+		return list_has_next_master(workers, it);
+	else if(it->possibly_parallel == 0)
+		return list_has_next_unblocked_worker(workers, it);
+
+	int nworkers = workers->nworkers;
+	STARPU_ASSERT(it != NULL);
+
+	unsigned ret = it->cursor < nworkers ;
+
+	if(!ret) it->cursor = 0;
+
+	return ret;
+}
+
+static int list_get_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+{
+	if(it->possibly_parallel == 1)
+		return list_get_next_master(workers, it);
+	else if(it->possibly_parallel == 0)
+		return list_get_next_unblocked_worker(workers, it);
+
+	int *workerids = (int *)workers->workerids;
+	int nworkers = (int)workers->nworkers;
+
+	STARPU_ASSERT(it->cursor < nworkers);
+
+	int ret = workerids[it->cursor++];
+
+	return ret;
+}
+
 static unsigned _worker_belongs_to_ctx(struct starpu_worker_collection *workers, int workerid)
 {
 	int *workerids = (int *)workers->workerids;
@@ -133,6 +166,9 @@ static int list_remove(struct starpu_worker_collection *workers, int worker)
 	int *workerids = (int *)workers->workerids;
 	unsigned nworkers = workers->nworkers;
 
+	int *unblocked_workers = (int *)workers->unblocked_workers;
+	unsigned nunblocked_workers = workers->nunblocked_workers;
+
 	int *masters = (int *)workers->masters;
 	unsigned nmasters = workers->nmasters;
 	
@@ -152,6 +188,21 @@ static int list_remove(struct starpu_worker_collection *workers, int worker)
 	if(found_worker != -1)
 		workers->nworkers--;
 
+	int found_unblocked = -1;
+	for(i = 0; i < nunblocked_workers; i++)
+	{
+		if(unblocked_workers[i] == worker)
+		{
+			unblocked_workers[i] = -1;
+			found_unblocked = worker;
+			break;
+		}
+	}
+
+	_rearange_workerids(unblocked_workers, nunblocked_workers);
+	if(found_unblocked != -1)
+		workers->nunblocked_workers--;
+
 	int found_master = -1;
 	for(i = 0; i < nmasters; i++)
 	{
@@ -182,15 +233,20 @@ static void _init_workers(int *workerids)
 static void list_init(struct starpu_worker_collection *workers)
 {
 	int *workerids;
+	int *unblocked_workers;
 	int *masters;
 
 	_STARPU_MALLOC(workerids, (STARPU_NMAXWORKERS+STARPU_NMAX_COMBINEDWORKERS) * sizeof(int));
+	_STARPU_MALLOC(unblocked_workers, (STARPU_NMAXWORKERS+STARPU_NMAX_COMBINEDWORKERS) * sizeof(int));
 	_STARPU_MALLOC(masters, (STARPU_NMAXWORKERS+STARPU_NMAX_COMBINEDWORKERS) * sizeof(int));
 	_init_workers(workerids);
+	_init_workers(unblocked_workers);
 	_init_workers(masters);
 
 	workers->workerids = (void*)workerids;
 	workers->nworkers = 0;
+	workers->unblocked_workers = (void*)unblocked_workers;
+	workers->nunblocked_workers = 0;
 	workers->masters = (void*)masters;
 	workers->nmasters = 0;
 
@@ -200,6 +256,7 @@ static void list_init(struct starpu_worker_collection *workers)
 static void list_deinit(struct starpu_worker_collection *workers)
 {
 	free(workers->workerids);
+	free(workers->unblocked_workers);
 	free(workers->masters);
 }
 
@@ -207,31 +264,47 @@ static void list_init_iterator(struct starpu_worker_collection *workers, struct
 {
 	(void) workers;
 	it->cursor = 0;
+	it->possibly_parallel = -1; /* -1 => we don't care about this field */
+
+}
+
+static void list_init_iterator_for_parallel_tasks(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it, struct starpu_task *task)
+{
+	list_init_iterator(workers, it);
+	if (_starpu_get_nsched_ctxs() <= 1)
+		return;
+
+	it->possibly_parallel = task->possibly_parallel; /* 0/1 => this field indicates if we consider masters only or slaves not blocked too */
 
 	int *workerids = (int *)workers->workerids;
 	unsigned nworkers = workers->nworkers;
 	unsigned i;
-	int nm = 0;
+	int nm = 0, nub = 0;
 	for(i = 0;  i < nworkers; i++)
 	{
-		if(!starpu_worker_is_slave(workerids[i]))
-			((int*)workers->masters)[nm++] = workerids[i];
+		if(!starpu_worker_is_blocked_in_parallel(workerids[i]))
+		{
+			((int*)workers->unblocked_workers)[nub++] = workerids[i];
+			if(!it->possibly_parallel) /* don't bother filling the table with masters we won't use it anyway */
+				continue;
+			if(!starpu_worker_is_slave_somewhere(workerids[i]))
+				((int*)workers->masters)[nm++] = workerids[i];
+		}
 	}
 	workers->nmasters = nm;
-
+	workers->nunblocked_workers = nub;
 }
 
-struct starpu_worker_collection worker_list =
+struct starpu_worker_collection starpu_worker_list =
 {
 	.has_next = list_has_next,
 	.get_next = list_get_next,
-	.has_next_master = list_has_next_master,
-	.get_next_master = list_get_next_master,
 	.add = list_add,
 	.remove = list_remove,
 	.init = list_init,
 	.deinit = list_deinit,
 	.init_iterator = list_init_iterator,
+	.init_iterator_for_parallel_tasks = list_init_iterator_for_parallel_tasks,
 	.type = STARPU_WORKER_LIST
 };
 
