@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2015,2017,2019                      CNRS
- * Copyright (C) 2013,2017                                Inria
- * Copyright (C) 2013-2017,2019                           Université de Bordeaux
- * Copyright (C) 2013                                     Corentin Salingue
+ * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -51,6 +49,12 @@ int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
+#elif STARPU_MAXNODES == 1
+/* Cannot register a disk */
+int main(int argc, char **argv)
+{
+	return STARPU_TEST_SKIPPED;
+}
 #else
 
 int dotest(struct starpu_disk_ops *ops, void *param)
@@ -59,17 +63,16 @@ int dotest(struct starpu_disk_ops *ops, void *param)
 	int ret;
 
 	/* limit main ram to force to push in disk */
-	setenv("STARPU_LIMIT_CPU_MEM", RAM, 1);
+	setenv("STARPU_LIMIT_CPU_NUMA_MEM", RAM, 1);
 
 	/* Initialize StarPU without GPU devices to make sure the memory of the GPU devices will not be used */
 	// Ignore environment variables as we want to force the exact number of workers
-	unsetenv("STARPU_NCUDA");
-	unsetenv("STARPU_NOPENCL");
-	unsetenv("STARPU_NMIC");
 	struct starpu_conf conf;
 	ret = starpu_conf_init(&conf);
 	if (ret == -EINVAL)
 		return EXIT_FAILURE;
+	conf.precedence_over_environment_variables = 1;
+	conf.ncpus = 1;
 	conf.ncuda = 0;
 	conf.nopencl = 0;
 	conf.nmic = 0;
